library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity controller is
  port ( 
    clk : in STD_LOGIC;                     -- main clock (100Mhz)
	 reset : in STD_LOGIC;                   -- reset signal (SW7)
	 irr : in STD_LOGIC;                     -- infrared receiver
	 irs : out STD_LOGIC;                    -- infrared sender
	 rvl : out STD_LOGIC;                    -- infrared receiver visible led
	 svl : out STD_LOGIC;                    -- infrared sender visible led
	 adr : out STD_LOGIC_VECTOR(5 downto 0); -- segment leds address bus
	 seg : out STD_LOGIC_VECTOR(7 downto 0); -- segment leds
	 ldx : out STD_LOGIC_VECTOR(14 downto 0) -- leds (LD14 - LD0)
  );
end controller;

architecture Behavioral of controller is

  component display
  port (
    clk : in  STD_LOGIC;
    reset : in  STD_LOGIC;
	 enable : in STD_LOGIC;
    disp1 : in  STD_LOGIC_VECTOR (7 downto 0);
    disp2 : in  STD_LOGIC_VECTOR (7 downto 0);
    disp3 : in  STD_LOGIC_VECTOR (7 downto 0);
    seg : out  STD_LOGIC_VECTOR (7 downto 0);
    adr : out  STD_LOGIC_VECTOR (5 downto 0)
  );
  end component;
  
  component filter
  port (
    clk : in STD_LOGIC;
    reset : in STD_LOGIC;
    ir_in : in STD_LOGIC;
    ir_out : out STD_LOGIC
  );
  end component;
  
  component modulator
  port (
    clk : in STD_LOGIC;
    reset : in STD_LOGIC;
    ir_in: in STD_LOGIC;
    ir_out : out STD_LOGIC
  );
  end component;
  
  component memory
  port ( 
    reset : in STD_LOGIC;
    store : in STD_LOGIC;
    data_in : in STD_LOGIC_VECTOR (31 downto 0);
	 address : in STD_LOGIC_VECTOR (1 downto 0);
	 data_out : out STD_LOGIC_VECTOR (31 downto 0)
  );
  end component;
  
  component decoder
  port (
    clk : in STD_LOGIC;
    reset : in STD_LOGIC;
	 enable : in STD_LOGIC;
	 nd : out STD_LOGIC;
	 data : out STD_LOGIC_VECTOR (31 downto 0);
    ir : in STD_LOGIC;
	 dsp_disp1 : out STD_LOGIC_VECTOR (7 downto 0); 
	 dsp_disp2 : out STD_LOGIC_VECTOR (7 downto 0);
	 dsp_disp3 : out STD_LOGIC_VECTOR (7 downto 0);
	 dsp_enable : out STD_LOGIC;
	 leds : OUT STD_LOGIC_VECTOR (14 downto 0)
  );
  end component;
  
  component encoder
  port (
    clk : in STD_LOGIC;
    reset : in STD_LOGIC;
    enable : in STD_LOGIC;
	 send : in STD_LOGIC;
    data : in STD_LOGIC_VECTOR (31 downto 0);
    ir : out STD_LOGIC;
	 dsp_disp1 : out STD_LOGIC_VECTOR (7 downto 0); 
	 dsp_disp2 : out STD_LOGIC_VECTOR (7 downto 0);
	 dsp_disp3 : out STD_LOGIC_VECTOR (7 downto 0);
	 dsp_enable : out STD_LOGIC;
	 leds : OUT STD_LOGIC_VECTOR (14 downto 0)
  );
  end component;
  
  component manager
  port ( 
    clk : in STD_LOGIC;
    reset : in STD_LOGIC;
	 dec_nd : in STD_LOGIC;
    dec_in : in STD_LOGIC_VECTOR (31 downto 0);
	 dec_enable : out STD_LOGIC;
	 enc_send : out STD_LOGIC;
	 enc_out : out STD_LOGIC_VECTOR (31 downto 0);
	 enc_enable : out STD_LOGIC;
	 mem_read : in STD_LOGIC_VECTOR (31 downto 0);
	 mem_write : out STD_LOGIC_VECTOR (31 downto 0);
	 mem_address : out STD_LOGIC_VECTOR (1 downto 0);
	 mem_store : out STD_LOGIC;
	 dsp_disp1 : out STD_LOGIC_VECTOR (7 downto 0);
	 dsp_disp2 : out STD_LOGIC_VECTOR (7 downto 0);
	 dsp_disp3 : out STD_LOGIC_VECTOR (7 downto 0);
	 dsp_enable : out STD_LOGIC;
	 leds : OUT STD_LOGIC_VECTOR (14 downto 0)
  );
  end component;
  
  signal dsp_disp1 : STD_LOGIC_VECTOR (7 downto 0);
  signal dsp_disp2: STD_LOGIC_VECTOR (7 downto 0);
  signal dsp_disp3 : STD_LOGIC_VECTOR (7 downto 0);
  signal dsp_enable : STD_LOGIC;
  signal dsp_disp1_d : STD_LOGIC_VECTOR (7 downto 0);
  signal dsp_disp2_d : STD_LOGIC_VECTOR (7 downto 0);
  signal dsp_disp3_d : STD_LOGIC_VECTOR (7 downto 0);
  signal dsp_enable_d : STD_LOGIC;
  signal dsp_disp1_e : STD_LOGIC_VECTOR (7 downto 0);
  signal dsp_disp2_e : STD_LOGIC_VECTOR (7 downto 0);
  signal dsp_disp3_e : STD_LOGIC_VECTOR (7 downto 0);
  signal dsp_enable_e : STD_LOGIC;
  signal dsp_disp1_s : STD_LOGIC_VECTOR (7 downto 0);
  signal dsp_disp2_s : STD_LOGIC_VECTOR (7 downto 0);
  signal dsp_disp3_s : STD_LOGIC_VECTOR (7 downto 0);
  signal dsp_enable_s : STD_LOGIC;
  
  signal leds_d : STD_LOGIC_VECTOR (14 downto 0);
  signal leds_e : STD_LOGIC_VECTOR (14 downto 0);
  signal leds_s : STD_LOGIC_VECTOR (14 downto 0);
  
  signal irr_f : STD_LOGIC;
  signal irs_p : STD_LOGIC;
  signal irs_n : STD_LOGIC;
  
  signal mem_out : STD_LOGIC_VECTOR (31 downto 0);
  signal mem_in : STD_LOGIC_VECTOR (31 downto 0);
  signal mem_address : STD_LOGIC_VECTOR (1 downto 0);
  signal mem_store : STD_LOGIC;
  
  signal	dec_nd : STD_LOGIC;
  signal dec_data : STD_LOGIC_VECTOR (31 downto 0);
  signal dec_enable : STD_LOGIC;
  
  signal	enc_send : STD_LOGIC;
  signal enc_data : STD_LOGIC_VECTOR (31 downto 0);
  signal enc_enable : STD_LOGIC;
  
begin

   display_impl: display
	port map (
	  clk => clk,
	  reset => reset,
	  enable => dsp_enable,
	  disp1 => dsp_disp1,
	  disp2 => dsp_disp2,
	  disp3 => dsp_disp3,
	  seg => seg,
	  adr => adr
	);
	
   filter_impl: filter
	port map (
	  clk => clk,
	  reset => reset,
	  ir_in => irr,
	  ir_out => irr_f
	);
	
	modulator_impl: modulator
	port map (
     clk => clk,
	  reset => reset,
	  ir_in => irs_p,
	  ir_out => irs_n
	);
	
	memory_impl: memory
	port map (
	  reset => reset,
	  store => mem_store,
	  data_in => mem_in,
	  address => mem_address,
	  data_out => mem_out
	);
	
	decoder_impl: decoder
	port map (
	  clk => clk,
	  reset => reset,
	  enable => dec_enable,
	  nd => dec_nd,
	  data => dec_data,
	  ir => irr_f,
	  dsp_disp1 => dsp_disp1_d,
	  dsp_disp2 => dsp_disp2_d,
	  dsp_disp3 => dsp_disp3_d,
	  dsp_enable => dsp_enable_d,
	  leds => leds_d
	);
	
	encoder_impl: encoder
	port map (
	  clk => clk,
	  reset => reset,
	  enable => enc_enable,
	  send => enc_send,
	  data => enc_data,
	  ir => irs_p,
	  dsp_disp1 => dsp_disp1_e,
	  dsp_disp2 => dsp_disp2_e,
	  dsp_disp3 => dsp_disp3_e,
	  dsp_enable => dsp_enable_e,
	  leds => leds_e
	);
	
   manager_impl: manager
	port map (
	  clk => clk,
     reset => reset,
	  dec_nd => dec_nd,
     dec_in => dec_data,
	  dec_enable => dec_enable,
	  enc_send => enc_send,
	  enc_out => enc_data,
	  enc_enable => enc_enable,
	  mem_read => mem_out,
	  mem_write => mem_in,
	  mem_address => mem_address,
	  mem_store => mem_store,
	  dsp_disp1 => dsp_disp1_s,
	  dsp_disp2 => dsp_disp2_s,
	  dsp_disp3 => dsp_disp3_s,
	  dsp_enable => dsp_enable_s,
	  leds => leds_s
	);
	
	ldx <= leds_e or leds_d or leds_s;
	dsp_disp1 <= dsp_disp1_e or dsp_disp1_d or dsp_disp1_s;
	dsp_disp2 <= dsp_disp2_e or dsp_disp2_d or dsp_disp2_s;
	dsp_disp3 <= dsp_disp3_e or dsp_disp3_d or dsp_disp3_s;
	dsp_enable <= dsp_enable_e or dsp_enable_d or dsp_enable_s;
	
	rvl <= not irr_f and dec_enable;
	svl <= irs_p and enc_enable;
	irs <= not irs_n;
	
end Behavioral;