library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

-- iterate over all segments to update them in
-- a PWM like fashion.  

entity iterator is
  port ( 
    clk : in STD_LOGIC;                            -- clock signal
	 reset : in STD_LOGIC;                          -- reset signal
    disp1 : in STD_LOGIC_VECTOR (7 downto 0);      -- byte display 1
    disp2 : in STD_LOGIC_VECTOR (7 downto 0);      -- byte display 2
    disp3 : in STD_LOGIC_VECTOR (7 downto 0);      -- byte display 3
    half_byte : out STD_LOGIC_VECTOR (3 downto 0); -- half byte data for single segment
    segment : out STD_LOGIC_VECTOR (2 downto 0)    -- segment number
  );
end iterator;

architecture Behavioral of iterator is

  signal c : INTEGER range 0 to 5 := 0;

begin

  -- step through all segments in a round robin fashion
  process (clk, reset)
  begin
    if reset = '1' then
	   c <= 0;
	 else
	   if clk = '1' and clk'Event then
		  if c= 5 then
		    c <= 0;
        else
		    c <= c + 1;
        end if;
	   end if;
	 end if;
  end process;
  
  -- the address is given by sel
  with c select
  segment <= 
    "000" when 0,
	 "001" when 1,
	 "010" when 2,
	 "011" when 3,
	 "100" when 4,
	 "101" when 5;
  
  -- select the right data segment
  with c select
  half_byte <= 
    disp3(3 downto 0) when 0,
	 disp3(7 downto 4) when 1,
	 disp1(3 downto 0) when 2,
	 disp1(7 downto 4) when 3,
	 disp2(3 downto 0) when 4,
	 disp2(7 downto 4) when 5;
	 
end Behavioral;