library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- creates a 38KHz modulated signal

entity modulator is
  port ( 
    clk : in STD_LOGIC;    -- main clock (100Mhz)
    reset : in STD_LOGIC;  -- reset signal
    ir_in : in STD_LOGIC;  -- input signal
    ir_out : out STD_LOGIC -- output signal
  );
end modulator;

architecture Behavioral of modulator is

  -- pulse length of on pulse at 38KHz
  constant pulse_length : INTEGER := 1316;

  signal c : INTEGER range 0 to 1316 := 0;
  signal clkmod : STD_LOGIC := '0';
  
begin

  -- swap the signal every 'pulse_length' many ticks
  process (clk, reset)
  begin
    if reset = '1' then 
		clkmod <= '0';
		c <= 0;
    else
	   if clk = '1' and clk'Event then
		  if c >= pulse_length then
		    clkmod <= not clkmod;
		    c <= 0;
		  else
		    c <= c + 1;
		  end if;
		end if;
  	 end if;
  end process;

  -- combine the input signal and the clock by AND
  ir_out <= '1' when ir_in = '1' and clkmod = '1' else '0';
  
end Behavioral;

